/* This program is Copyright (c) 1990 David Allen.  It may be freely
   distributed as long as you leave my name and copyright notice on it.
   I'd really like your comments and feedback; send e-mail to
   davea@vlsi.ll.mit.edu, or send us-mail to David Allen, 10 O'Moore Ave,
   Maynard, MA 01754. */

/* This file contains the function getparams(), which reads parameters
   from an input file.  The default values for all of the parameters are
   set here as well. */

#include "const.h"
#include <stdio.h>

/* These are all of the adjustable parameters; they are completely
   described in the file params.doc. */

long XSIZE = 90,	YSIZE = 90,	MAXSTEP = 100;
long MAXBUMP = 50,	BUMPTOL = 50;
long DRAWEVERY = 1,	BLOBLEVEL = 64,	DRAWMODE = DRAWMODE_TEXT;
long ZINIT = 22,	ZSUBSUME = 16,	ZCOAST = 16;
long ZSHELF = 8,	ZMOUNTAIN = 48;
long RIFTPCT = 40,	DOERODE = 1,	ERODERND = 4;
long MAXCTRTRY = 50,	RIFTDIST = 5,	BENDEVERY = 6;
long BENDBY = 50,	SPEEDRNG = 300,	SPEEDBASE = 200;

double MR [] = { 1.0,1.0,1.0,0.7,0.4,0.1,-0.2,-0.5,-0.8,-1.0 };
long MAXLIFE = 10; /* Length of MR vector */

FILE *fp;

char getbuf[256]; /* Storage for each token as it is being read */
char in[32];

extern float atof ();
double nextdouble();
long nextlong();
int nextword(char *);

#define MIN(a,b)   ((a) < (b) ? (a) : (b))
#define MAX(a,b)   ((a) > (b) ? (a) : (b))

getparams (s) char *s; { 
   /* This function is called by init() with one parameter, a string which
   comes directly off the command line.  If the string is null, getparams()
   is never called; otherwise, a "-" means standard input, while any other
   text is treated as a filename and handed to fopen().  If the file doesn't
   exist the program exits via panic().  Once the file is open, the function
   expects to find paren-delimited pairs of (name value).  Value could be
   a vector, like (name (val1 val2)).  After a name is found, there is
   a very simple test, one string comparison per adjustable parameter,
   to see if the name is recognized.  If unrecognized, the program panics.
   If it is a recognized name, the appropriate parsing function, with the
   appropriate parameters, is called.  There are three parsing functions,
   getdim(), getlng(), and getdvec(). */

   int at_end,i;
   long newval;

   if (!strcmp (s, "-")) {
      fp = stdin;
   } else {
      fprintf(stderr,"Opening params file: %s\n",s);
      fp = fopen (s, "r");
      if (!fp) panic ("Can't find input file");
   }
   
   while (!(at_end = nextword(getbuf))) {
      /* fprintf(stderr,"at_end equals (%d)\n",at_end); */
      if (!strncmp(getbuf,"XSIZE",2)) {
         newval = nextlong();
         XSIZE = MIN(MAXX,newval);
      } else if (!strncmp(getbuf,"YSIZE",2)) {
         newval = nextlong();
         YSIZE = MIN(MAXY,newval);
      } else if (!strncmp(getbuf,"MAXSTEP",4)) MAXSTEP = nextlong();
      else if (!strncmp(getbuf,"MAXBUMP",4)) MAXBUMP = nextlong();
      else if (!strncmp(getbuf,"BUMPTOL",2)) BUMPTOL = nextlong();
      else if (!strncmp(getbuf,"DRAWEVERY",5)) DRAWEVERY = nextlong();
      else if (!strncmp(getbuf,"DRAWMODE",5)) DRAWMODE = nextlong();
      else if (!strncmp(getbuf,"BLOBLEVEL",2)) BLOBLEVEL = nextlong();
      else if (!strncmp(getbuf,"ZINIT",2)) ZINIT = nextlong();
      else if (!strncmp(getbuf,"ZSUBSUME",3)) ZSUBSUME = nextlong();
      else if (!strncmp(getbuf,"ZCOAST",2)) ZCOAST = nextlong();
      else if (!strncmp(getbuf,"ZSHELF",3)) ZSHELF = nextlong();
      else if (!strncmp(getbuf,"ZMOUNTAIN",2)) ZMOUNTAIN = nextlong();
      else if (!strncmp(getbuf,"RIFTPCT",5)) RIFTPCT = nextlong();
      else if (!strncmp(getbuf,"RIFTDIST",5)) RIFTDIST = nextlong();
      else if (!strncmp(getbuf,"DOERODE",2)) DOERODE = nextlong();
      else if (!strncmp(getbuf,"ERODERND",1)) ERODERND = nextlong();
      else if (!strncmp(getbuf,"MAXCTRTRY",4)) MAXCTRTRY = nextlong();
      else if (!strncmp(getbuf,"BENDEVERY",5)) BENDEVERY = nextlong();
      else if (!strncmp(getbuf,"BENDBY",5)) BENDBY = nextlong();
      else if (!strncmp(getbuf,"SPEEDBASE",6)) SPEEDBASE = nextlong();
      else if (!strncmp(getbuf,"SPEEDRNG",6)) SPEEDRNG = nextlong();
      else if (!strncmp(getbuf,"MOVERATE",2)) {
         /* first number is int, the number of entries in the array! */
         MAXLIFE = nextlong();
         for (i=0; i<MAXLIFE; i++) MR[i] = nextdouble();
      } else if (!strncmp(getbuf,"end",3)) break;
      else geterr ("unknown parameter name");
      /* fprintf(stderr,"done with that option...\n"); */
   }  /* end while */

   fclose(fp);
}

int nextword(char *in) {
  char ch;
  int i = 0;

  while(isspace(ch = fgetc(fp))) { if(ch == EOF) return(1); }
  in[0] = ch;
  while(!isspace(ch = fgetc(fp))) { 
    in[++i] = ch;
  }
  in[++i] = '\0';
  /* fprintf(stderr,"Read a string (%s)\n",in); */
  return(0);
}

long nextlong() {
  char ch;
  int i = 0;

  while(isspace(ch = fgetc(fp))) { }
  in[0] = ch;
  while(!isspace(ch = fgetc(fp))) {
    in[++i] = ch;
  }
  in[++i] = '\0';
  /* fprintf(stderr,"Read a long (%s)\n",in); */
  return(atol(in));
}

double nextdouble() {
  char ch;
  int i = 0;

  while(isspace(ch = fgetc(fp))) { }
  in[0] = ch;
  while(!isspace(ch = fgetc(fp))) {
    in[++i] = ch;
  }
  in[++i] = '\0';
  /* fprintf(stderr,"Read a double (%s)\n",in); */
  return(atof(in));
}

geterr (s) char *s; {
   /* Calls machine-specific panic() with nicely formatted message */
   sprintf (getbuf, "Parameter error: %s", s);
   panic (getbuf);
}
